subroutine calcmiso(ndisk,Rdisk,Sigmad,ap,Mstar,Misodisk,Misolate)

IMPLICIT NONE
INCLUDE 'units.h90'

INTEGER,INTENT(IN)                                ::ndisk
DOUBLE PRECISION,INTENT(IN)                       ::ap,Mstar
DOUBLE PRECISION,DIMENSION(ndisk),INTENT(IN)      ::Rdisk,Sigmad
DOUBLE PRECISION,INTENT(OUT)                      ::Misodisk,Misolate
INTEGER                                           ::ip,i
DOUBLE PRECISION,PARAMETER                        ::fliss=5D0  !half width of the feeding zone in Hill spheres. Must be the same as in sbr evodiskd.
LOGICAL,PARAMETER                                 ::DEBUG=.FALSE.
DOUBLE PRECISION,PARAMETER                        ::rhoe=1D0   !mean density of the planet. Must be the same as in sbr evodiskd.

IF(DEBUG)THEN
   WRITE(*,*)
   WRITE(*,*)'SBR Calcmiso'
END IF

!get the solid surface density at the planet's position
ip=0
DO i=1,ndisk-1
   IF(Rdisk(i)<=ap.AND.Rdisk(i+1)>=ap)THEN
      ip=i
      IF(DEBUG)WRITE(*,*)ip,ap/AU,Rdisk(ip)/AU,sigmad(ip)
      EXIT
   END IF
END DO
IF(ip==0)STOP'ERROR: SBR MDOTCORE: Planet not in disk'

!Calculate the isolation mass during the presence of the gas disk
Misodisk=(4D0*PI*fliss*ap**2D0*Sigmad(ip))**1.5D0/(3D0*Mstar)**0.5D0
!Calculate the isolation mass after the dissipation of the gas disk
Misolate=4D0*PI**(7D0/4D0)*rhoe**0.25*Sigmad(ip)**1.5D0*ap**(15D0/4D0)/(3D0**0.25D0*Mstar**0.75D0)

IF(DEBUG)THEN
   WRITE(*,*)'Isolation mass during gas disk/Mearth',Misodisk/Mearth
   WRITE(*,*)'Isolation mass after gas disk/Mearth ',Misolate/Mearth
   !READ(*,*)
END IF

RETURN
end subroutine
