PROGRAM driver

!(c) Christoph Mordasini, University of Bern, 2015
!driver for toy model of planet formation
!three modes of operation: single planet, systematic study varying the initial position, populatioin synthesis

IMPLICIT NONE
INCLUDE 'units.h90'

!Initial conditions/input
LOGICAL                            ::AFF,writeTrack
INTEGER                            ::iplanet
DOUBLE PRECISION                   ::Mstar,Rindisk,Routdisk,feh,fg,taudisk,ap
DOUBLE PRECISION                   ::pT,alpha,fpgsun,kappa,pKH,qKH,C1,C2,CMdotc
INTEGER                            ::ilimMe,itypeI,itypeII

!Mode of operating
INTEGER                            ::imode,icont

iplanet=0
AFF=.TRUE.

WRITE(*,*)'Mode of operation: single planet (1), systematic study (2), population synthesis (3)'
READ(*,*)imode

!which mode of operation
IF(imode==1)THEN
   !write formation tracks in this mode
   writeTrack=.TRUE. 
   DO 
      !read initial condition
      CALL initcond(Mstar,Rindisk,Routdisk,feh,fg,taudisk,ap)
      iplanet=iplanet+1

      !read the parameters
      CALL getparams(pT,alpha,fpgsun,kappa,pKH,qKH,ilimMe,itypeI,C1,itypeII,C2,CMdotc)

      !start the global formation and evolution model
      CALL globalPFE(writeTrack,AFF,iplanet,Mstar,Rindisk,Routdisk,feh,fg,taudisk,ap,&
           pT,alpha,fpgsun,kappa,pKH,qKH,ilimMe,itypeI,C1,itypeII,C2,CMdotc)

      !stop or continue with another simulation?
      !WRITE(*,*)
      !WRITE(*,*)'Read another initial condition (1) or stop (0)'
      !READ(*,*)icont
      icont=0
      CLOSE(79)
      IF(icont==0)STOP
   END DO

ELSE IF(imode==2)THEN   !vary only the semimajor axis, keeping other initial conditions constant
   !do not write formation tracks in this mode
   writeTrack=.FALSE. 

   !but write global output files at specific times  
   OPEN(95,file='ref_red1e5.dat')
   OPEN(96,file='ref_red1e6.dat')
   OPEN(97,file='ref_red1e7.dat')
   OPEN(98,file='ref_red1e8.dat')
   OPEN(99,file='ref_red1e9.dat')
   OPEN(100,file='ref_red1e10.dat')

   !read the initial conditions
   CALL initcond(Mstar,Rindisk,Routdisk,feh,fg,taudisk,ap)

   !read the parameters
   CALL getparams(pT,alpha,fpgsun,kappa,pKH,qKH,ilimMe,itypeI,C1,itypeII,C2,CMdotc)

   !in this mode we vary the initial semimajor axis from 0.1 to 100 AU. Loop over them.
   DO iplanet=1,301
      !set the semimajor axis uniform in log
      ap=10D0**(-1D0+REAL(iplanet-1)/100D0)*AU
  
      !lunch the global formation and evolution model
      CALL globalPFE(writeTrack,AFF,iplanet,Mstar,Rindisk,Routdisk,feh,fg,taudisk,ap,&
           pT,alpha,fpgsun,kappa,pKH,qKH,ilimMe,itypeI,C1,itypeII,C2,CMdotc)
   END DO

ELSE  !run population synthesis
      CALL popsynth
END IF

WRITE(*,*)'Program terminated: May the Entropy be with you'
END PROGRAM driver
