SUBROUTINE evodiskd(time,dt,Mstar,ndisk,Rdisk,Sigmad,Mddisk,Mdotc,Mc,Me,ap,Sigmadmean,gasdisk,Mfeed)
                   
!(c) Christoph Mordasini, University of Bern, 2015
!evolve the gas disk

IMPLICIT NONE
INCLUDE 'units.h90'
INTEGER,INTENT(IN)                                ::ndisk
LOGICAL,INTENT(IN)                                ::gasdisk
DOUBLE PRECISION,INTENT(IN)                       ::time,dt,Mdotc,Mc,Me,ap,Mstar
DOUBLE PRECISION,DIMENSION(ndisk),INTENT(IN)      ::Rdisk
DOUBLE PRECISION,DIMENSION(ndisk),INTENT(INOUT)   ::Sigmad
DOUBLE PRECISION,INTENT(OUT)                      ::Mddisk,Sigmadmean,Mfeed
LOGICAL,PARAMETER                                 ::DEBUG=.FALSE.
INTEGER                                           ::i,ip,imin,imax,iplotmin,iplotmax
DOUBLE PRECISION                                  ::Mp,Rmin,Rmax,Rhills,Rp,Vsurf,OmegaK
DOUBLE PRECISION,PARAMETER                        ::fliss=6D0  !half width of the feeding zone in Hill spheres. Must be the same as in sbr calcmiso.
DOUBLE PRECISION,PARAMETER                        ::rhoe=1D0   !mean density of the planet. Must be the same as in sbr mdotcore.

IF(DEBUG)THEN
   WRITE(*,*)
   WRITE(*,*)'SBR EVODISK'
END IF

!total mass of planet
Mp=Mc+Me

!make some output
IF(DEBUG)WRITE(*,*)time/an,Sigmad(1),Sigmad(ndisk),Mc/Mearth,Me/Mearth,ap/AU,gasdisk

!-------------------------
!width of the feeding zone
!-------------------------
IF(gasdisk)THEN
   !during the presence of the gas disk: half width is fliss*Rhills
   Rmin=ap-fliss*Rhills(Mp,Mstar,ap)
   Rmax=ap+fliss*Rhills(Mp,Mstar,ap)
ELSE
   !after dissipatio of the gas disk: half width is vsurf/Omega_K
   Rp=(3D0*Mp/(4D0*rhoe))**(1D0/3D0)
   Vsurf=SQRT(G*Mp/Rp)
   OmegaK=SQRT(G*Mstar/ap**3D0)
   Rmin=ap-(0.5D0*Vsurf/OmegaK)
   Rmax=ap+(0.5D0*Vsurf/OmegaK)   
END IF

IF(DEBUG)WRITE(*,*)'Orig Rmin, Rmax',Rmin/AU,Rmax/AU

Rmin=MAX(Rmin,Rdisk(2))
Rmax=MIN(Rmax,Rdisk(ndisk-1))

IF(DEBUG)WRITE(*,*)'Corr Rmin, Rmax',Rmin/AU,Rmax/AU


!--------------------------
!limits of the feeding zone
!--------------------------
DO i=1,ndisk-1

   IF(Rdisk(i)<=Rmin.AND.Rdisk(i+1)>Rmin)THEN
      imin=i
   END IF

   IF(Rdisk(i)<=Rmax.AND.Rdisk(i+1)>Rmax)THEN
      imax=i
   END IF 

   IF(Rdisk(i)<=ap.AND.Rdisk(i+1)>ap)THEN
      ip=i
   END IF

END DO

!small planets which have a too small feeding zone for the grid planets
!artificially set feeding zone width to two grid cells
imax=MAX(imax,imin+2)

IF(DEBUG)WRITE(*,*)ip,ap/AU,Rdisk(ip)/AU,sigmad(ip),imax-imin
IF(DEBUG)WRITE(*,*)imin,Rmin/AU,Rdisk(imin)/AU,sigmad(imin)
IF(DEBUG)WRITE(*,*)imax,Rmax/AU,Rdisk(imax)/AU,sigmad(imax)

!------------------------
!Mass in the feeding zone
!------------------------
Mfeed=0D0
Do i=imin,imax-1
   Mfeed=Mfeed+PI*(Rdisk(i+1)**2D0-Rdisk(i)**2D0)*0.5D0*(Sigmad(i+1)+Sigmad(i))
END DO

IF(DEBUG)WRITE(*,*)Mfeed/Mearth,-dt*Mdotc/Mearth

!------------------------------------
!subtract mass accreted by the planet
!------------------------------------
Mfeed=Mfeed-dt*Mdotc
Mfeed=max(Mfeed,0D0)

!------------------------------------------------
!uniform mean surface density in the feeding zone
!------------------------------------------------
Sigmadmean=Mfeed/(PI*(Rdisk(imax)**2D0-Rdisk(imin)**2D0))
IF(Sigmadmean<1D-20)Sigmadmean=0D0

Do i=imin,imax
   Sigmad(i)=Sigmadmean
END DO

IF(DEBUG)WRITE(*,*)Mfeed/Mearth,Sigmadmean

!---------------------------------------
!Calculate the total disk mass of solids
!---------------------------------------
Mddisk=0D0
DO i=1,ndisk-1
   Mddisk=Mddisk+PI*(Rdisk(i+1)**2D0-Rdisk(i)**2D0)*0.5D0*(Sigmad(i+1)+Sigmad(i))
END DO


IF(DEBUG)THEN
   iplotmin=MAX(imin-20,1)
   iplotmax=MIN(imax+20,ndisk)

   DO i=iplotmin,iplotmax
      WRITE(22,'(100ES14.4)')time/an,Rdisk(i)/AU,Sigmad(i)
   END DO

   WRITE(23,'(100G15.5)')time/an,imin,Rmin/AU,Rdisk(imin)/AU,ip,ap/AU,Rdisk(ip)/AU,imax,Rmax/AU,Rdisk(imax)/AU,imax-imin,&
        sigmad(imin),sigmad(ip),sigmad(imax),Sigmadmean,Mfeed/Mearth,dt*Mdotc/Mearth,Mc/Mearth,Me/Mearth
END IF

!READ(*,*)

RETURN
END SUBROUTINE evodiskd
