!*************************
! driver for interpolation
! (c) C. Mordasini, Unibe
! ************************

subroutine drivequadradisk(npoints_rad,r_disque,yvect,Rloc,yloc,yderi)

IMPLICIT NONE
INCLUDE 'units.h90'

DOUBLE PRECISION,DIMENSION(10000)  ::Yvect,R_disque
INTEGER                            ::npoints_rad   
DOUBLE PRECISION                   ::Rloc
DOUBLE PRECISION                   ::Yloc,Yderi
DOUBLE PRECISION,DIMENSION(npoints_rad)                  ::Yvectloc,R_disqueloc
LOGICAL,PARAMETER                            ::DEBUG=.FALSE.
DOUBLE PRECISION                   ::Rinf,Rsup,aa,bb,cc
INTEGER                            ::k,i,n_app,ordre

IF(npoints_rad>10000)THEN
   WRITE(*,*)'drivequadra: npoints_rad>nrad_max ', npoints_rad
   WRITE(*,*)'PROBLEME: drivequadra vector too long'
   STOP
END IF
      
Rinf = Rloc * (1. - 0.1)
Rsup = Rloc * (1. + 0.1)
Rinf = max(Rinf,R_disque(2))
Rsup = min(Rsup,R_disque(npoints_rad))

IF(DEBUG)THEN
   WRITE(*,*)
   WRITE(*,*)'SBR DRIVEQUADRADISK'
   WRITE(*,*)'Rinf,Rloc,Rsup',Rinf/AU,Rloc/AU,Rsup/AU
END IF
    

k = 1
n_app = 0
ordre = 1
do i=1,npoints_rad
   if((R_disque(i).ge.Rinf).and.(R_disque(i).le.Rsup))then
      Yvectloc(k) =  Yvect(i)
      R_disqueloc(k) =  R_disque(i)
      IF(DEBUG)WRITE(*,*)k, R_disqueloc(k)/AU, Yvectloc(k)
      k = k+1
      n_app = n_app + 1
   end if
end do

call interpolationquadra(Yvectloc,R_disqueloc,n_app,aa,bb,cc,ordre)
      
yderi=bb
yloc=bb*Rloc+cc
 
      
IF(DEBUG)WRITE(*,*)'yloc,yderi',yloc,yderi

RETURN
END subroutine drivequadradisk




subroutine interpolationquadra(X,t,N,a,b,c,ordre)

!***********************
! SBR for interpolation
! (c) Y. Alibert, Unibe
! **********************

! interpolation par un polynoime de degre ordre (1 ou 2) 
! utilisee par APPROX Q
! entree : X,t : tableaux de taille N, ordre
! sortie : a,b,c
! ordre 1 : X = bt + c
! ordre 2 X =0.5*a*t**2. + bt + c

IMPLICIT NONE

DOUBLE PRECISION,DIMENSION(10000)  ::X,t
DOUBLE PRECISION                   ::a,b,c
INTEGER                            ::N,ordre
DOUBLE PRECISION                   ::alpha,beta,gamma,delta,lambda,mu
DOUBLE PRECISION                   ::sig1,sig2,sig3,sig4,sigX,sigXt1,sigXt2
INTEGER                            ::i
  
if(N.gt.10000)then
   write(*,*) 'Tableaux trop grands dans interpolation quadra'
   write(*,*) 'PROBLEME : INTERPOLATION QUADRA'
   stop
end if

if(ordre.eq.1)then
   sig1 = 0.
   sig2 = 0.
   sigX = 0.
   sigXt1 = 0.
   
   do i=1,N
      sig1 = sig1 + t(i)
      sig2 = sig2 + t(i)**2.
      sigX = sigX + X(i)
      sigXt1 = sigXt1 + X(i)*t(i)
   end do

   alpha = sig2
   beta  = sig1
   gamma = sig1
   delta = N
   lambda = sigXt1
   mu = sigX

   a = 0.
   b = (lambda*delta-beta*mu)/(alpha*delta-gamma*beta)
   c = (lambda*gamma-mu*alpha)/(beta*gamma-delta*alpha)
   
   return

end if


if(ordre.eq.2)then
   sig1 = 0.
   sig2 = 0.
   sig3 = 0.
   sig4 = 0.
   sigX = 0.
   sigXt1 = 0.
   sigXt2 = 0.
   
   do i=1,N
      sig1 = sig1 + t(i)
      sig2 = sig2 + t(i)**2.
      sig3 = sig3 + t(i)**3.
      sig4 = sig4 + t(i)**4.
      sigX = sigX + X(i)
      sigXt1 = sigXt1 + X(i)*t(i)
      sigXt2 = sigXt2 + X(i)*t(i)**2.
   end do
      
   alpha  = N*sig4 - sig2**2.
   gamma  = N*sig3 -sig2*sig1
   beta   = 2.*N*sig3-2.*sig2*sig1
   delta  = 2.*N*sig2-2.*sig1**2.
   
   lambda = 2.*N*sigXt2 - 2. * sigX*sig2
   mu     = 2.*N*sigXt1 - 2. * sigX*sig1
   
   a = (lambda*delta-beta*mu)/(alpha*delta-gamma*beta)
   b = (lambda*gamma-mu*alpha)/(beta*gamma-delta*alpha)
   c = sigX/N - 1./(2.*N) * sig2 * a - 1./N * sig1 * b
   
end if

return
end subroutine interpolationquadra



