SUBROUTINE Mdotcore(time,dt,Mstar,ndisk,Rdisk,Sigmag,Mc,Me,ap,Mdotc,Sigmadmean,gasdisk,Mfeed,CMdotc)

!(c) Christoph Mordasini, University of Bern, 2015
!calculate the core accretion rate

IMPLICIT NONE
INCLUDE 'units.h90'
INTEGER,INTENT(IN)                                ::ndisk
LOGICAL,INTENT(IN)                                ::gasdisk
DOUBLE PRECISION,INTENT(IN)                       ::time,dt,ap,Mstar,Me,Sigmadmean,Mfeed,CMdotc
DOUBLE PRECISION,DIMENSION(ndisk),INTENT(IN)      ::Rdisk,Sigmag
DOUBLE PRECISION,INTENT(INOUT)                    ::Mc,Mdotc  
INTEGER                                           ::i,ip
LOGICAL,PARAMETER                                 ::DEBUG=.false.
DOUBLE PRECISION                                  ::tauc,McisoIL,McisoMA,Mciso
DOUBLE PRECISION,PARAMETER                        ::mpla=1D18,fcap=0.1D0
DOUBLE PRECISION,PARAMETER                        ::rhoe=1D0   !mean density of the planet. Must be the same as in sbr evodiskd.
DOUBLE PRECISION                                  ::taue,Meiso,Mesca,Rp,vsurf,vesc
DOUBLE PRECISION,PARAMETER                        ::Sigmagmin=1D0  !must be the same as in sbr evodiskg

IF(DEBUG)THEN
   WRITE(*,*)
   WRITE(*,*)'SBR Mdotcore'
END IF

!get the gas surface density at the planet's position
ip=0
DO i=1,ndisk-1
   IF(Rdisk(i)<=ap.AND.Rdisk(i+1)>=ap)THEN
      ip=i
      IF(DEBUG)WRITE(*,*)ip,ap/AU,Rdisk(ip)/AU,sigmag(ip)
      EXIT
   END IF
END DO
IF(ip==0)STOP'ERROR: SBR MDOTCORE: Planet not in disk'

!--------------------------------------------------------------------------------
!calculate the accretion rate during the presence of the gas disk with IL04 Eq. 6
!--------------------------------------------------------------------------------
tauc=1.2D5*an*(Sigmadmean/10D0)**(-1D0)*(ap/AU)**0.5*((Mc+Me)/Mearth)**(1D0/3D0)*(Mstar/Msol)**(-1D0/6D0)*&
     ((Sigmag(ip)/2.4D3)**(-1D0/5D0)*(ap/AU)**(1D0/20D0)*(mpla/1D18)**(1D0/15D0))**2D0

!----------------------------------------------------------------------------------
!calculate the accretion rate after the dissipation of the gas disk with IL04 Eq. 9
!----------------------------------------------------------------------------------
taue=2D7*an*(Sigmadmean/10D0)**(-1D0)*((Mc+Me)/Mearth)**(1D0/3D0)*(rhoe/1D0)**(2D0/3D0)*&
     (Mstar/Msol)**(-0.5D0)*(ap/AU)**1.5D0
!smooth the beginning of growth at sigmagmin for numerical reasons
taue=taue*(1D0+10D0*(Sigmag(ip)/sigmagmin))

!----------------------------
!take into account scattering
!----------------------------
Rp=(3D0*(Mc+Me)/(4D0*rhoe))**(1D0/3D0)
Vsurf=SQRT(G*(Mc+Me)/Rp)
Vesc=SQRT(2D0*G*Mstar/ap)
Taue=taue*(1D0+(Vsurf/Vesc)**4D0)

!-------------------
!core accretion rate
!-------------------
IF(gasdisk)THEN
   mdotc=CMdotc*Mc/tauc
ELSE
   mdotc=CMdotc*Mc/taue
END IF

IF(DEBUG)WRITE(*,*)'Mdotcore/Mearth*yr',mdotc/Mearth*an

!------------------------------------------------------------
!calculate the isolation mass during the presence of the disk
!------------------------------------------------------------
McisoIL=0.16D0*Mearth*(Sigmadmean/10D0)**1.5D0*(ap/AU)**3*(Mstar/Msol)**(-0.5D0) !IL04 Eq. 20
McisoMA=(4D0*PI*5D0*ap**2D0*Sigmadmean)**1.5D0/(3D0*Mstar)**0.5D0
!IF(DEBUG)WRITE(*,*)McisoIL/Mearth,McisoMA/Mearth
Mciso=McisoIL

!-----------------------
!increment the core mass
!-----------------------
!cannot accrete more than mass in the feeding zone
Mc=Mc+MIN(dt*mdotc,Mfeed)


IF(DEBUG)WRITE(79,'(100ES15.5)')time/an,Sigmadmean,sigmag(ip),Mc/Mearth,Me/Mearth,Mciso/Mearth,Meiso/Mearth,Mesca/Mearth,tauc/an,taue/an,mdotc/Mearth*an,(Vsurf/Vesc)**4D0

RETURN
END SUBROUTINE Mdotcore
