SUBROUTINE Mdotgas(time,dt,Mstar,ndisk,Rdisk,Sigmag,Hdisk,Mc,Me,ap,Mdotc,alpha,kappa,pKH,qKH,ilimMe,Mdote,Mgdisk)

!(c) Christoph Mordasini, University of Bern, 2015 
!calculate the envelope accretion rate and final gas mass

IMPLICIT NONE
INCLUDE 'units.h90'
INTEGER,INTENT(IN)                                ::ndisk,ilimMe
DOUBLE PRECISION,INTENT(IN)                       ::time,dt,ap,Mstar,Mc,Mdotc,Mgdisk,alpha,kappa,pKH,qKH
DOUBLE PRECISION,DIMENSION(ndisk),INTENT(IN)      ::Rdisk,Sigmag,Hdisk
DOUBLE PRECISION,INTENT(INOUT)                    ::Me,Mdote  
INTEGER                                           ::i,ip
LOGICAL,PARAMETER                                 ::DEBUG=.FALSE.
DOUBLE PRECISION                                  ::Mcrit,tauKH,Meiso,Mdotdisk,nudisk,mdotbondi,Mp,Rhills,fva04
DOUBLE PRECISION,PARAMETER                        ::flub=0.9D0

IF(DEBUG)THEN
   WRITE(*,*)
   WRITE(*,*)'SBR Mdotgas'
END IF

ip=0

!get the disk properties at the planet's position
DO i=1,ndisk-1
   IF(Rdisk(i)<=ap.AND.Rdisk(i+1)>=ap)THEN
      ip=i
      IF(DEBUG)WRITE(*,*)ip,ap/AU,Rdisk(ip)/AU,sigmag(ip),Hdisk(ip)/AU
      EXIT
   END IF
END DO
IF(ip==0)STOP'ERROR: SBR MDOTGAS: Planet not in disk'

!total planet mass 
Mp=Mc+Me

!calculate the critical core mass
Mcrit=10D0*Mearth*(Mdotc/(1e-6*Mearth/an))**0.25*(kappa/1D0)**0.25


!calculate the Kelvin Helmholtz timescale
IF(Mc>Mcrit)THEN
   tauKH=10D0**pKH*an*(Mp/Mearth)**qKH*(kappa/1D0)
   mdote=Mp/tauKH
ELSE
   tauKH=1D10*an
   mdote=0D0
END IF

!cannot accrete more than at the Bondi rate
mdotbondi=sigmag(ip)/Hdisk(ip)*SQRT(G*Mstar/ap**3)*(Rhills(Mp,Mstar,ap)/3D0)**3D0
mdote=MIN(mdote,mdotbondi)

!calculate the disk accretion rate and limit gas accretion to that
nudisk=alpha*Hdisk(ip)**2D0*SQRT(G*Mstar/ap**3D0)
Mdotdisk=flub*3D0*PI*nudisk*Sigmag(ip)
mdote=MIN(mdote,mdotdisk)

!---------------------------------------
!various ways to terminate the accretion
!---------------------------------------

!1) by the gas isolation mass
IF(ilimMe==1)THEN

   !calculate the gas isolation mass. Assume width of feeding zone is 2 RH as in IL04
   Meiso=(4D0*PI*2D0*ap**2D0*Sigmag(ip))**1.5D0/(3D0*Mstar)**0.5D0

   !limit Mdot at the isolation mass
   IF(Me>=Meiso)THEN
      mdote=0d0
      IF(DEBUG)WRITE(*,*)'Gas isolation mass reached',Meiso/Mearth
   END IF

!2) if the Hills sphere is larger than H (gap opening)
ELSE IF(ilimMe==2)THEN
   
   !limit Mdot at the gap opening mas
   IF(Hdisk(ip)<Rhills(Mp,Mstar,ap))THEN
      mdote=0d0
      IF(DEBUG)WRITE(*,*)'Gap opened',Mp/Mearth,Rhills(Mp,Mstar,ap)/AU,Hdisk(ip)/AU
   END IF

!3) if the Hills sphere is larger than 1.5H (gap opening)
ELSE IF(ilimMe==3)THEN
   
   !limit Mdot at the gap opening mas
   IF(1.5D0*Hdisk(ip)<Rhills(Mp,Mstar,ap))THEN
      mdote=0d0
      IF(DEBUG)WRITE(*,*)'Gap opened (1.5 RH)',Mp/Mearth,Rhills(Mp,Mstar,ap)/AU,1.5D0*Hdisk(ip)/AU
   END IF

!4) Don't limit at all (just limited by the disk lifetime)
ELSE IF(ilimMe==4)THEN

   !stop if created a star
   IF(Mp>0.1*Msol)THEN
      mdote=0D0
      IF(DEBUG)WRITE(*,*)'Star created',Mp/Mearth
   END IF

!5) Limit due to gap formation as in Veras & Armitage 2004
ELSE IF(ilimMe==5)THEN

   !limit only if created a star
   fva04=MIN(flub,1.668D0*(mp/Mj)**(1D0/3D0)*EXP(-Mp/(1.5D0*Mj))+0.04)
   Mdotdisk=fva04*3D0*PI*nudisk*Sigmag(ip)   
   mdote=MIN(mdote,mdotdisk)

ELSE

   STOP'ERROR: unknown ilimMe'
   
END IF

!----------------------------------------------------------------------
!global limit: allow no planets more massive than the current disk mass
!----------------------------------------------------------------------
IF(Me>Mgdisk)THEN
   mdote=0D0
   IF(DEBUG)WRITE(*,*)'Enve more massive than disk gas mass',Me/Mearth,Mgdisk/Mearth
END IF

!increment the gas mass
Me=Me+mdote*dt

IF(DEBUG)WRITE(*,*)'Mcrit/Mearth,tauKH,mdote',Mcrit/Mearth,tauKH/an,mdote/Mearth*an,Meiso/Mearth

RETURN
END SUBROUTINE Mdotgas
