subroutine migration(time,dt,Mstar,ndisk,Rdisk,Sigmag,Hdisk,Mc,Me,taudisk,ap,Rindisk,Routdisk,alpha,&
     itypeI,C1,itypeII,C2,pT,dlsigmagdlr,adot)
           
!(c) Christoph Mordasini, University of Bern, 2015
!calculate the orbital migration due to interaction with the gas disk. Currently only type II

IMPLICIT NONE
INCLUDE 'units.h90'
INTEGER,INTENT(IN)                                ::ndisk,itypeI,itypeII
DOUBLE PRECISION,INTENT(IN)                       ::time,dt,Mc,Me,Mstar,taudisk,Rindisk,Routdisk,alpha,C1,C2,pT
DOUBLE PRECISION,DIMENSION(ndisk),INTENT(IN)      ::Rdisk,Sigmag,Hdisk
DOUBLE PRECISION,INTENT(INOUT)                    ::ap
DOUBLE PRECISION,INTENT(OUT)                      ::dlsigmagdlr
LOGICAL,PARAMETER                                 ::DEBUG=.FALSE.
DOUBLE PRECISION                                  ::Rm,Simgmagm,OmegaKm,OmegaKp,adot,Mp,Rhills,nup,ur,tauI,yloc,yderi
DOUBLE PRECISION                                  ::Cadia,Cloc,CHSadia,Clind
INTEGER                                           ::im,ip,i
DOUBLE PRECISION,PARAMETER                        ::gamma=1.4D0

yderi=0D0
adot=0D0
Mp=Mc+Me

IF(DEBUG)THEN
   WRITE(*,*)
   WRITE(*,*)'SBR MIGRATION'
   WRITE(*,*)'time/yr   ', time/an
   WRITE(*,*)'taud/yr   ', taudisk/an
   WRITE(*,*)'alpha     ', alpha
   WRITE(*,*)'itypeI,II ', itypeI,itypeII
   WRITE(*,*)'C1,C2     ', C1,C2
END IF
  

!we follow IL04 Sect. 5.1

!estimate radius of maximum viscous couple/velocity reversal
Rm=10D0*AU*EXP(MIN(2D0*time/(5D0*taudisk),10D0))
Rm=MIN(Rm,Rdisk(ndisk))

!get disk properties at Rm
im=0
DO i=1,ndisk-1
   IF(Rdisk(i)<=Rm.AND.Rdisk(i+1)>=rm)THEN
      im=i
      IF(DEBUG)WRITE(*,*)im,Rm/AU,Rdisk(im)/AU,sigmag(im),Hdisk(im)/AU
      EXIT
   END IF
END DO
IF(im==0)THEN
   im=ndisk
   Rm=Rdisk(ndisk)
   IF(DEBUG)WRITE(*,*)'Rm outside disk',Rm/AU
END IF

!get disk properties at ap
ip=0
DO i=1,ndisk-1
   IF(Rdisk(i)<=ap.AND.Rdisk(i+1)>=ap)THEN
      ip=i
      IF(DEBUG)WRITE(*,*)ip,ap/AU,Rdisk(ip)/AU,sigmag(ip),Hdisk(ip)/AU
      EXIT
   END IF
END DO

!Keplerian frequency at planet position
OmegaKp=SQRT(G*Mstar/ap**3D0)

!viscosity at planet position
nup=alpha*Hdisk(ip)**2D0*OmegaKp

!velocity of the gas
ur=3D0*nup/(2D0*ap)

!Omega at Rm
OmegaKm=SQRT(G*Mstar/Rm**3D0)

!-----------------------------------------------------------
!Type I and Type II migration according to different authors
!-----------------------------------------------------------

IF(Hdisk(ip)>Rhills(Mp,Mstar,ap))THEN  !Type I migration
 
   !get local power law exponent of Sigmag
   IF(Sigmag(ip)>0D0)THEN
      call drivequadradisk(Ndisk,Rdisk,Sigmag,ap,yloc,yderi)
      dlsigmagdlr=yderi*ap/Sigmag(ip)
   ELSE
      yloc=0D0
      yderi=0D0
   END IF

   IF(itypeI==1)THEN !IL08 Eq.12
      IF(Sigmag(ip)>0D0)THEN
         tauI=1D0/(2.728+1.082*dlsigmagdlr)*(Hdisk(ip)/ap)**2D0*(Mstar/Mp)*(Mstar/(ap**2D0*Sigmag(ip)))*(1D0/OmegaKp)
         adot=-C1*ap/tauI
      ELSE
         adot=0D0
      END If

   ELSE IF(itypeI==2)THEN !Dittkrist et al. 2014 locally isothermal Eq. 7
      IF(Sigmag(ip)>0D0)THEN
         Cloc=-0.85+0.9*pT+dlsigmagdlr
         adot=C1*Cloc*2D0*ap**5D0*OmegaKp*Sigmag(ip)*Mp/(Hdisk(ip)**2D0*Mstar**2D0)
      ELSE
         adot=0D0
      END If

   ELSE IF(itypeI==3)THEN !Dittkrist et al. 2014 adiabatic Eq. 4-6
      IF(Sigmag(ip)>0D0)THEN
         Clind=1D0/gamma*(-2.5D0+1.7D0*pT-0.1D0*dlsigmagdlr)
         CHSadia=1D0/gamma*(1.65+dlsigmagdlr*(9D0-7.9D0/gamma)-7.9D0*pT/gamma)
         Cadia=Clind+CHSadia
         adot=C1*Cadia*2D0*ap**5D0*OmegaKp*Sigmag(ip)*Mp/(Hdisk(ip)**2D0*Mstar**2D0)
      ELSE
         adot=0D0
      END If

   ELSE
      STOP'UNKNOWN itypeI'
   END IF

ELSE

   !1) As in IL04
   IF(itypeII==1)THEN
      !Equation 64 and assume type II migration if Rhills>Hdisk
      adot=C2*SIGN(1D0,ap-Rm)*3D0*alpha*(Sigmag(im)*Rm**2D0/Mp)*(OmegaKm/OmegaKp)*(Hdisk(im)/ap)**2D0*OmegaKm*ap
 
   !2) As in Alibert et al. 2005, Eq 19
   ELSE IF(itypeII==2)THEN
      adot=C2*SIGN(1D0,ap-Rm)*ur*MIN(1D0,2D0*Sigmag(ip)*ap**2D0/Mp)
   ELSE
      STOP'UNKNOWN itypeII'
   END IF
END IF
   

!increment the semimajor axis
ap=ap+dt*adot

ap=MAX(ap,Rindisk)
ap=MIN(ap,Rdisk(ndisk))

!make some output in debbuging mode
IF(DEBUG)THEN
   WRITE(*,*)'Omega m,p ',OmegaKm,OmegaKp
   WRITE(*,*)'Sigmag(im)',Sigmag(im)
   WRITE(*,*)'Hdisk(im) ',Hdisk(im)/AU
   WRITE(*,*)'yloc,yderi',yloc,yderi
   WRITE(*,*)'Migration rate / AU / year',adot/AU*an
   IF(adot.NE.0D0)WRITE(*,*)'Migration timescale / year',ABS(ap/adot/an)
END IF

RETURN
end subroutine migration
