def readSyntheticPopulation(params):
  directory=params['popsynthDir']
  if directory[-1]<>'/':
    directory=directory+'/'
  filename=params['popsynthFile']

  print 'loading synthetic population:',directory+filename
  try:
    file=open(directory+filename,'r')
  except:
    print 'NOTE: synthetic population file not found'
    return 

  fieldList=['itime','time',      # timestep #; time (years)
             'Mc','Me','Mp',      # core/env/total mass (Earth masses)
             'Mgdisk','Mddisk',   # disk mass gas/solid (Earth masses)
             'Mdotc','Mdote',     # core/env accretion rate (Earth/yr)
             'ap',                # semi-major axis (AU)
             'dt',                # timestep (years)
             'sigmag','sigmad',   # gas/dust density at planet
             'Hdisk',             # disk scale height (AU)
             'Rhills',            # Hill sphere (AU)
             'dlsigmagdlr',       # local slope of gas density
             'adot',              # migration rate (AU/yr)
             'iplanet',           # planet number
             'mstar',             # star mass (M_Sun)
             'fg',                # gas disk scaling factor
             'Fe/H',              # metalicity
             'taudisk',           # disk decay timescale (years)
             'aice',              # iceline position (AU)
             'apini',             # initial planet position (AU)
             'Rindisk','Routdisk']# inner/outer disk radius (AU)

  synthPop={}
  for field in fieldList:
    synthPop[field]=[]
    
  for line in file:
    fields=line.split()
#    print '# of fields',len(fields),fields[0]
    if len(fields)<>len(fieldList):
      exit('ERROR: fields misaligned; edit fieldList to match input file')

    for fieldName,fieldValue in zip(fieldList,fields):
      if fieldValue[-4:]=='-312':
#        print 'NOTE: fortran output problem with small float', \
#              fieldName,fieldValue
        fieldValue='0'
      synthPop[fieldName].append(float(fieldValue))

  if len(synthPop['ap'])<>len(synthPop['Mp']):
    print 'ERROR: synthetic data arrays have different lengths'
    
#  print '  finished reading in the simulation planet data'
  print '  # of simulated planets',len(synthPop['Mp'])
  print  
  return synthPop
#______________________________________________________________________________
  
def readArchiveTables(params):
  directory=params['archiveDir']
  if directory[-1]<>'/':
    directory=directory+'/'
  confirmedFilename=params['confirmedDataFile']
  keplerFilename=params['keplerDataFile']
  
  print 'loading real planet data from directory:',directory

# FIRST, load in the confirmed planet table

# optionally select a subset of the possible columns
#  (drop the variable or leave it blank for all columns)
  fieldList=params['confirmedDataFields']
#  fieldList=[]
  confirmedPlanets=readConfirmedPlanets(directory,confirmedFilename, \
                                        desiredFields=fieldList)
#  print 'confirmed exoplanet data successfully read in'
#  print ' # of fields: ',len(confirmedPlanets)
#  print ' # of planets:',len(confirmedPlanets['pl_hostname'])


# SECOND, load in the Kepler planet table

# optionally select a subset of the possible columns
#  (drop the variable or leave it blank for all columns)
  fieldList=params['keplerDataFields']
#  fieldList=[]
  KeplerList=readKeplerPlanets(directory,keplerFilename, \
                               desiredFields=fieldList)
#  print 'kepler exoplanet data successfully read in'
#  print ' # of fields: ',len(KeplerList)
#  print ' # of planets:',len(KeplerList['kepler_name'])

#  print 'M sin i (Mjup)',confirmedPlanets['pl_msinij']
#  print 'M sin i (Mearth)',confirmedPlanets['pl_msinie']

#  print 'finished reading in the real planet data'
  print '  # confirmed, # kepler',len(confirmedPlanets['pl_msinij']), \
        len(KeplerList['kepler_name'])
  print

  return confirmedPlanets,KeplerList
#______________________________________________________________________________

def readConfirmedPlanets(dir,filename, desiredFields=None):
  '''
  Download exoplanet data from the exoplanet archive.
  If data has already been downloaded, just open the saved file.

  Inputs:
  filename = name of the csv file (either an existing file or one to be created here)
  desiredFields = optional argument for selecting a subset of the possible columns
    (blank value means all data is returned)
  '''
  import csv,shutil

#  defaultfilename=filename+'.defaultColumns'
#  if os.access(dir+filename, os.R_OK):
  try:
    file=open(dir+filename,'r')
    print ' using pre-downloaded confirmed planet file:',filename
  except:
    print ' '+filename,'does not exist'
    print ' downloading new data from archive:',filename
    os.system('wget "http://exoplanetarchive.ipac.caltech.edu/cgi-bin/nstedAPI/nph-nstedAPI?table=exoplanets&format=csv" -O '+dir+filename)
    file=open(dir+filename,'r')
  csvfile=csv.reader(file)    

  try:
    header=next(csvfile)
  except:
    exit('ERROR: the file is blank.  Delete it!')
  
# if the desired fields are not passed in, select everything
  if not desiredFields:
    desiredFields=header

# check that all of the desired field exist in this data
  missingfields=[]
  for field in desiredFields:
    if not field in header:
      print 'WARNING: desired field does not exit',field
      missingfields.append(field)

# reload the archive data, with specified (non-default) columns
  if missingfields:
    print 'NOTE: some desired columns are not in the default table:',missingfields
    print ' --> re-downloading with specified columns as:',filename
    selectedColumns=''
    for field in desiredFields:
      selectedColumns=selectedColumns+field+' '
    print 'selectedColumns',selectedColumns
    os.system('wget "http://exoplanetarchive.ipac.caltech.edu/cgi-bin/nstedAPI/nph-nstedAPI?table=exoplanets&format=csv&select='+selectedColumns+'" -O '+dir+filename)
    file=open(dir+filename,'r')
    csvfile=csv.reader(file)
    header=next(csvfile)
          
  data={}
  for field in desiredFields:
    data[field]=[]
  for line in csvfile:
    for field,value in zip(header,line):
      if field in desiredFields:
          try:
            data[field].append(float(value))
          except:
            data[field].append(value)

# Do some validation of the archive data
  for mass,name in zip(data['pl_msinij'],
                       data['pl_hostname']):
    if mass==0.:
      pass

  for inc,trans,astrom,type,name in zip(data['pl_orbincl'],
                                        data['pl_tranflag'],
                                        data['pl_astflag'],
                                        data['pl_discmethod'],
                                        data['pl_hostname']):
    if trans:
      if inc:
        if float(inc)<75. or float(inc)>90.:
          pass
#          print 'NOTE: odd inclination for transiting planet:',inc,name
      else:
        pass
# these values simply aren't in the published data. ah well.
#        print 'TROUBLE: no inclination for transit planet',inc,name
    else:
      if inc:
        pass
#        print 'NOTE: inclination for non-transiting planet:',inc,type,name,astrom

# check for msini/m consistency
  for msini,mass,type,inc,trans,letter,name in zip(data['pl_msinij'],
                                                   data['pl_massj'],
                                                   data['pl_discmethod'],
                                                   data['pl_orbincl'],
                                                   data['pl_tranflag'],
                                                   data['pl_letter'],
                                                   data['pl_hostname']):
    if type=='Radial Velocity':
      if not msini:
        pass
#        print 'ERROR: no msini for an RV planet',name,letter,trans,inc
#      else:
#        print 'ok, it has an m-sin-i value',name
# o.k. this check below is essentially the same as above
      if mass and not inc:
        if trans:
          pass
#          print 'ERROR: its transiting, but no inclination',name,letter
#        print 'NOTE: RV-discovered planet has Mass:',name,letter
    elif 'Transit' in type:
      if msini:
        pass
#        print 'NOTE: there is msini for a Transit planet',name,letter

# some transit guys do have mass, some don't.  not so informative
#      if mass:
#        print 'NOTE: transiting planet with a mass',name,letter
#      else:
#        print 'NOTE: transiting planet without a mass',name,letter

# hmm, o.k. these all seem to be multiple-planet systems, 
#  so that must be why they're confirmed without RV
#      if not mass:        
#        print 'Transit planet without mass:',name,letter
        
    elif type=='Imaging':
      pass
    elif type=='Microlensing':
      pass
    else:
      pass
#      print 'type',type

  for msinij,msinie,letter,name in zip(data['pl_msinij'],
                                       data['pl_msinie'],
#                                       data['pl_discmethod'],
#                                       data['pl_orbincl'],
#                                       data['pl_tranflag'],
                                       data['pl_letter'],
                                       data['pl_hostname']):
    if msinij:
      if msinie:
        if msinie/msinij<317. or msinie/msinij>319.:
          pass
# o.k. this just spots some rounding errors
#          print 'mass ratio',msinie/msinij,name,letter,msinie,msinij
      else:
        print 'ERROR: jup mass, but not earth mass',name,letter
    elif msinie:
# o.k. this just spots the above HD57274 problem (zero Jupiter mass)
      pass
#      print 'ERROR: earth mass, but not jup mass',name,letter

  for field in desiredFields:
    if not data[field]:
      print 'WARNING: desired field not properly filled in:',field
#    else:
#      print 'OK: field filled in:',field

#  exit('done loading and checking the data')
  return data
#______________________________________________________________________________

def readKeplerPlanets(dir,filename, desiredFields=None):
  '''
  Download exoplanet data from the exoplanet archive.
  If data has already been downloaded, just open the saved file.
  Inputs:
  filename = name of the csv file (either an existing file or one to be created here)
  desiredFields = optional argument for selecting a subset of the possible columns
    (blank value means all data is returned)
  '''
  import csv,shutil

  try:
    file=open(dir+filename,'r')
    print ' using pre-downloaded kepler planet file:',filename
  except:
    candString="'CANDIDATE'"
    print ' '+filename,'does not exist'
    print ' downloading new data from archive:',filename    
    os.system('wget "http://exoplanetarchive.ipac.caltech.edu/cgi-bin/nstedAPI/nph-nstedAPI?table=cumulative&where=koi_pdisposition like '+candString+'&format=csv" -O '+dir+filename)
    file=open(dir+filename,'r')
  csvfile=csv.reader(file)

  try:
    header=next(csvfile)
  except:
    exit('ERROR: the file is blank.  Delete it!')
  
# if the desired fields are not passed in, select everything
  if not desiredFields:
    desiredFields=header

# check that all of the desired field exist in this data
  missingfields=[]
  for field in desiredFields:
#    print 'checking field',field
    if not field in header:
      print 'WARNING: desired field does not exist:',field
      missingfields.append(field)

# reload the archive data, with specified (non-default) columns
  if missingfields:
    print 'NOTE: some desired columns are not in the default table:',missingfields
    print ' --> re-downloading with specified columns as:',filename
    selectedColumns=''
    for field in desiredFields:
      selectedColumns=selectedColumns+field+' '
    print 'selectedColumns',selectedColumns
# make sure to include the koi_pdisposition clause
#  or you get a bunch of 'FALSE POSITIVES'
    candString="'CANDIDATE'"
    os.system('wget "http://exoplanetarchive.ipac.caltech.edu/cgi-bin/nstedAPI/nph-nstedAPI?table=cumulative&where=koi_pdisposition like '+candString+'&format=csv&select='+selectedColumns+'" -O '+dir+filename)

    file=open(dir+filename,'r')
    csvfile=csv.reader(file)
    header=next(csvfile)    
      
  data={}
  for field in desiredFields:
    data[field]=[]
  for line in csvfile:

# ignore all the false positives
#  6/8/15 this keeps only 4664 out of 8826
# no longer needed, since the wget was modified above
#  but might as well leave it in here just as a cross-check
    if 'FALSE POSITIVE' in line:
      print 'dropping a false positive from kepler list'
    else:
      lineinfo=dict(zip(header,line))

# optional conditional to drop kepler planets without radius
      if 0:
#      if (not lineinfo['koi_prad'] or lineinfo['koi_prad']=='0') \
#             and lineinfo['koi_disposition']=='CANDIDATE':
#        print 'TROUBLE: undefined planet:',lineinfo
        pass
      else:
#        print 'KEEPING: defined planet:',lineinfo

# this finds Kepler-295 d with blank radius.  must be dealt with later
#        if not lineinfo['koi_prad']:
#          print 'NOTE: undefined candidate confirmed?',lineinfo
        for field,value in zip(header,line):
          if field in desiredFields:
            try:
              data[field].append(float(value))
            except:
              data[field].append(value)

# Do some validation of the archive data
  if 0:
    for mass,name in zip(data['pl_msinij'],
                         data['pl_hostname']):
      if mass==0.:
        pass
#      print 'TROUBLE: zero planet mass!',name

    for inc,type,name in zip(data['pl_orbincl'],
                             data['pl_discmethod'],
                             data['pl_hostname']):
      if inc:
        if float(inc)<75. or float(inc)>90.:
          pass
          print 'NOTE: odd inclination for transiting planet:',inc,name
      else:
# these values simply aren't in the published data. ah well.
        print 'TROUBLE: no inclination for transit planet',inc,name

# check for msini/m consistency
    for msini,mass,type,inc,letter,name in zip(data['pl_msinij'],
                                               data['pl_massj'],
                                               data['pl_discmethod'],
                                               data['pl_orbincl'],
                                               data['pl_letter'],
                                               data['pl_hostname']):
      if type=='Radial Velocity':
        if not msini:
          pass
#          print 'ERROR: no msini for an RV planet',name,letter,inc
#        else:
#          print 'ok, it has an m-sin-i value',name
# o.k. this check below is essentially the same as above
        if mass and not inc:
          print 'ERROR: its transiting, but no inclination',name,letter
      elif 'Transit' in type:
        if msini:
          pass
#          print 'NOTE: there is msini for a Transit planet',name,letter
 
    for msinij,letter,name in zip(data['pl_msinij'],
                                  data['pl_letter'],
                                  data['pl_hostname']):
      pass

    for field in desiredFields:
      if not data[field]:
        print 'WARNING: desired field not properly filled in:',field
#      else:
#        print 'OK: field filled in:',field
    
#  exit('done loading and checking the data')
  return data
#______________________________________________________________________________

